@@ -1,26 +1,18 @@
 /*  -*- C++ -*-
     This file is part of the KDE libraries
-    Copyright (C) 1997 Tim D. Gilman (tdgilman@best.org)
-              (C) 1998-2001 Mirko Boehm (mirko@kde.org)
-              (C) 2007 John Layt <john@layt.net>
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    SPDX-FileCopyrightText: 1997 Tim D. Gilman <tdgilman@best.org>
+    SPDX-FileCopyrightText: 1998-2001 Mirko Boehm (mirko@kde.org)
+    SPDX-FileCopyrightText: 2007 John Layt <john@layt.net>
+    SPDX-FileCopyrightText: 2022 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kdatepicker.h"
 #include "kdatepicker_p.h"
+
+#include "common_helpers_p.h"
 #include "kdatetable_p.h"
 #include <kpopupframe.h>
 
@@ -28,58 +20,74 @@
 #include <QComboBox>
 #include <QFont>
 #include <QFontDatabase>
-#include <QLayout>
 #include <QKeyEvent>
+#include <QLayout>
 #include <QMenu>
-#include <QPainter>
 #include <QStyle>
 #include <QToolButton>
-#include <QDoubleValidator>
-#include <QFontDatabase>
 
+#include "loggingcategory.h"
 #include "moc_kdatepicker.cpp"
 #include "moc_kdatepicker_p.cpp"
 
 class DatePickerValidator : public QValidator
 {
+    Q_OBJECT
 public:
-    DatePickerValidator(KDatePicker *parent)
-        : QValidator(parent), picker(parent) {}
-
-    State validate(QString &text, int &) const Q_DECL_OVERRIDE
+    explicit DatePickerValidator(KDatePicker *parent)
+        : QValidator(parent)
+        , picker(parent)
     {
-        QLocale::FormatType formats[] = { QLocale::LongFormat, QLocale::ShortFormat, QLocale::NarrowFormat };
-        QLocale locale = picker->locale();
+    }
 
-        for (int i = 0; i < 3; i++) {
-            QDate tmp = locale.toDate(text, formats[i]);
-            if (tmp.isValid()) {
-                return Acceptable;
+    State validate(QString &text, int &) const override
+    {
+        return toDate(text).isValid() ? QValidator::Acceptable : QValidator::Intermediate;
+    }
+
+    QDate toDate(const QString &text) const
+    {
+        QLocale locale = picker->locale();
+        const QList<QString> formats{
+            locale.dateFormat(QLocale::LongFormat),
+            locale.dateFormat(QLocale::ShortFormat),
+            locale.dateFormat(QLocale::NarrowFormat),
+            dateFormatWith4DigitYear(locale, QLocale::ShortFormat),
+            QStringLiteral("yyyy-MM-dd"),
+        };
+
+        QDate date;
+        for (const auto &format : formats) {
+            date = locale.toDate(text, format);
+            if (date.isValid()) {
+                break;
             }
         }
-
-        return QValidator::Intermediate;
+        if (!date.isValid()) {
+            qCDebug(KWidgetsAddonsLog) << "Could not parse text as date:" << text;
+        }
+        return date;
     }
 
 private:
-    KDatePicker *picker;
+    KDatePicker *const picker;
 };
 
 // Week numbers are defined by ISO 8601
 // See http://www.merlyn.demon.co.uk/weekinfo.htm for details
 
-KDatePickerPrivateYearSelector::KDatePickerPrivateYearSelector(
-    const QDate &currentDate, QWidget *parent)
-    : QLineEdit(parent), val(new QIntValidator(this)), result(0)
+KDatePickerPrivateYearSelector::KDatePickerPrivateYearSelector(const QDate &currentDate, QWidget *parent)
+    : QLineEdit(parent)
+    , val(new QIntValidator(this))
+    , result(0)
+    , oldDate{currentDate}
 {
-    oldDate = currentDate;
-
     setFont(QFontDatabase::systemFont(QFontDatabase::GeneralFont));
 
     setFrame(false);
 
     // TODO: Find a way to get that from QLocale
-    //val->setRange( calendar->year( calendar->earliestValidDate() ),
+    // val->setRange( calendar->year( calendar->earliestValidDate() ),
     //               calendar->year( calendar->latestValidDate() ) );
     setValidator(val);
 
@@ -101,11 +109,10 @@ void KDatePickerPrivateYearSelector::yearEnteredSlot()
     // check if new year will lead to a valid date
     if (QDate(newYear, oldDate.month(), oldDate.day()).isValid()) {
         result = newYear;
-        emit(closeMe(1));
+        Q_EMIT closeMe(1);
     } else {
         QApplication::beep();
     }
-
 }
 
 int KDatePickerPrivateYearSelector::year()
@@ -113,16 +120,18 @@ int KDatePickerPrivateYearSelector::year()
     return result;
 }
 
-void KDatePickerPrivateYearSelector::setYear(int year)
+void KDatePickerPrivateYearSelector::setYear(const QDate &year)
 {
-    setText(QString::number(year));
+    setText(locale().toString(year, QStringLiteral("yyyy")).rightJustified(4, QLatin1Char('0')));
 }
 
-class Q_DECL_HIDDEN KDatePicker::KDatePickerPrivate
+class KDatePickerPrivate
 {
+    Q_DECLARE_TR_FUNCTIONS(KDatePicker)
+
 public:
-    KDatePickerPrivate(KDatePicker *q) :
-        q(q), closeButton(nullptr), selectWeek(nullptr), todayButton(nullptr), navigationLayout(nullptr)
+    explicit KDatePickerPrivate(KDatePicker *qq)
+        : q(qq)
     {
     }
 
@@ -132,37 +141,37 @@ public:
     /// the date table
     KDatePicker *q;
 
-    QToolButton *closeButton;
-    QComboBox *selectWeek;
-    QToolButton *todayButton;
-    QBoxLayout *navigationLayout;
+    QToolButton *closeButton = nullptr;
+    QComboBox *selectWeek = nullptr;
+    QToolButton *todayButton = nullptr;
+    QBoxLayout *navigationLayout = nullptr;
 
     /// the year forward button
-    QToolButton *yearForward;
+    QToolButton *yearForward = nullptr;
     /// the year backward button
-    QToolButton *yearBackward;
+    QToolButton *yearBackward = nullptr;
     /// the month forward button
-    QToolButton *monthForward;
+    QToolButton *monthForward = nullptr;
     /// the month backward button
-    QToolButton *monthBackward;
+    QToolButton *monthBackward = nullptr;
     /// the button for selecting the month directly
-    QToolButton *selectMonth;
+    QToolButton *selectMonth = nullptr;
     /// the button for selecting the year directly
-    QToolButton *selectYear;
+    QToolButton *selectYear = nullptr;
     /// the line edit to enter the date directly
-    QLineEdit *line;
+    QLineEdit *line = nullptr;
     /// the validator for the line edit:
-    DatePickerValidator *val;
+    DatePickerValidator *val = nullptr;
     /// the date table
-    KDateTable *table;
+    KDateTable *table = nullptr;
     /// the widest month string in pixels:
     QSize maxMonthRect;
 
     /// the font size for the widget
-    int fontsize;
+    int fontsize = -1;
 };
 
-void KDatePicker::KDatePickerPrivate::fillWeeksCombo()
+void KDatePickerPrivate::fillWeeksCombo()
 {
     // every year can have a different number of weeks
     // it could be that we had 53,1..52 and now 1..53 which is the same number but different
@@ -181,12 +190,11 @@ void KDatePicker::KDatePickerPrivate::fillWeeksCombo()
     // adding an entry to the week combo for each week in the year
 
     for (; day.isValid() && day <= lastDayOfYear; day = day.addDays(7)) {
-
         // Get the ISO week number for the current day and what year that week is in
         // e.g. 1st day of this year may fall in week 53 of previous year
         int weekYear = thisYear;
         const int week = day.weekNumber(&weekYear);
-        QString weekString = tr("Week %1").arg(QString::number(week));
+        QString weekString = tr("Week %1").arg(q->locale().toString(week));
 
         // show that this is a week from a different year
         if (weekYear != thisYear) {
@@ -200,15 +208,15 @@ void KDatePicker::KDatePickerPrivate::fillWeeksCombo()
 
         // make sure that the week of the lastDayOfYear is always inserted: in Chinese calendar
         // system, this is not always the case
-        if (day < lastDayOfYear &&
-                day.daysTo(lastDayOfYear) < 7 &&
-                lastDayOfYear.weekNumber() != day.weekNumber()) {
+        if (day < lastDayOfYear //
+            && day.daysTo(lastDayOfYear) < 7 //
+            && lastDayOfYear.weekNumber() != day.weekNumber()) {
             day = lastDayOfYear.addDays(-7);
         }
     }
 }
 
-QDate KDatePicker::KDatePickerPrivate::validDateInYearMonth(int year, int month)
+QDate KDatePickerPrivate::validDateInYearMonth(int year, int month)
 {
     QDate newDate;
 
@@ -225,52 +233,55 @@ QDate KDatePicker::KDatePickerPrivate::validDateInYearMonth(int year, int month)
     return newDate;
 }
 
-KDatePicker::KDatePicker(QWidget *parent) : QFrame(parent), d(new KDatePickerPrivate(this))
+KDatePicker::KDatePicker(QWidget *parent)
+    : QFrame(parent)
+    , d(new KDatePickerPrivate(this))
 {
     initWidget(QDate::currentDate());
 }
 
 KDatePicker::KDatePicker(const QDate &date_, QWidget *parent)
-    : QFrame(parent), d(new KDatePickerPrivate(this))
+    : QFrame(parent)
+    , d(new KDatePickerPrivate(this))
 {
     initWidget(date_);
 }
 
 void KDatePicker::initWidget(const QDate &date_)
 {
-    const int spacingHint = style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
+    const int horizontalSpacing = style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing);
 
     QBoxLayout *topLayout = new QVBoxLayout(this);
     topLayout->setSpacing(0);
-    topLayout->setMargin(0);
+    topLayout->setContentsMargins(0, 0, 0, 0);
 
     d->navigationLayout = new QHBoxLayout();
     d->navigationLayout->setSpacing(0);
-    d->navigationLayout->setMargin(0);
+    d->navigationLayout->setContentsMargins(0, 0, 0, 0);
     topLayout->addLayout(d->navigationLayout);
     d->navigationLayout->addStretch();
     d->yearBackward = new QToolButton(this);
     d->yearBackward->setAutoRaise(true);
     d->navigationLayout->addWidget(d->yearBackward);
     d->monthBackward = new QToolButton(this);
-    d->monthBackward ->setAutoRaise(true);
+    d->monthBackward->setAutoRaise(true);
     d->navigationLayout->addWidget(d->monthBackward);
-    d->navigationLayout->addSpacing(spacingHint);
+    d->navigationLayout->addSpacing(horizontalSpacing);
 
     d->selectMonth = new QToolButton(this);
-    d->selectMonth ->setAutoRaise(true);
+    d->selectMonth->setAutoRaise(true);
     d->navigationLayout->addWidget(d->selectMonth);
     d->selectYear = new QToolButton(this);
     d->selectYear->setCheckable(true);
     d->selectYear->setAutoRaise(true);
     d->navigationLayout->addWidget(d->selectYear);
-    d->navigationLayout->addSpacing(spacingHint);
+    d->navigationLayout->addSpacing(horizontalSpacing);
 
     d->monthForward = new QToolButton(this);
-    d->monthForward ->setAutoRaise(true);
+    d->monthForward->setAutoRaise(true);
     d->navigationLayout->addWidget(d->monthForward);
     d->yearForward = new QToolButton(this);
-    d->yearForward ->setAutoRaise(true);
+    d->yearForward->setAutoRaise(true);
     d->navigationLayout->addWidget(d->yearForward);
     d->navigationLayout->addStretch();
 
@@ -286,19 +297,19 @@ void KDatePicker::initWidget(const QDate &date_)
 
     d->fontsize++; // Make a little bigger
 
-    d->selectWeek = new QComboBox(this);    // read only week selection
+    d->selectWeek = new QComboBox(this); // read only week selection
     d->selectWeek->setFocusPolicy(Qt::NoFocus);
     d->todayButton = new QToolButton(this);
     d->todayButton->setIcon(QIcon::fromTheme(QStringLiteral("go-jump-today")));
 
-    d->yearForward->setToolTip(tr("Next year"));
-    d->yearBackward->setToolTip(tr("Previous year"));
-    d->monthForward->setToolTip(tr("Next month"));
-    d->monthBackward->setToolTip(tr("Previous month"));
-    d->selectWeek->setToolTip(tr("Select a week"));
-    d->selectMonth->setToolTip(tr("Select a month"));
-    d->selectYear->setToolTip(tr("Select a year"));
-    d->todayButton->setToolTip(tr("Select the current day"));
+    d->yearForward->setToolTip(tr("Next year", "@info:tooltip"));
+    d->yearBackward->setToolTip(tr("Previous year", "@info:tooltip"));
+    d->monthForward->setToolTip(tr("Next month", "@info:tooltip"));
+    d->monthBackward->setToolTip(tr("Previous month", "@info:tooltip"));
+    d->selectWeek->setToolTip(tr("Select a week", "@info:tooltip"));
+    d->selectMonth->setToolTip(tr("Select a month", "@info:tooltip"));
+    d->selectYear->setToolTip(tr("Select a year", "@info:tooltip"));
+    d->todayButton->setToolTip(tr("Select the current day", "@info:tooltip"));
 
     // -----
     setFontSize(d->fontsize);
@@ -316,13 +327,13 @@ void KDatePicker::initWidget(const QDate &date_)
         d->monthBackward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
     }
 
-    connect(d->table, SIGNAL(dateChanged(QDate)), SLOT(dateChangedSlot(QDate)));
+    connect(d->table, &KDateTable::dateChanged, this, &KDatePicker::dateChangedSlot);
     connect(d->table, &KDateTable::tableClicked, this, &KDatePicker::tableClickedSlot);
     connect(d->monthForward, &QAbstractButton::clicked, this, &KDatePicker::monthForwardClicked);
     connect(d->monthBackward, &QAbstractButton::clicked, this, &KDatePicker::monthBackwardClicked);
     connect(d->yearForward, &QAbstractButton::clicked, this, &KDatePicker::yearForwardClicked);
     connect(d->yearBackward, &QAbstractButton::clicked, this, &KDatePicker::yearBackwardClicked);
-    connect(d->selectWeek, SIGNAL(activated(int)), SLOT(weekSelected(int)));
+    connect(d->selectWeek, qOverload<int>(&QComboBox::activated), this, &KDatePicker::weekSelected);
     connect(d->todayButton, &QAbstractButton::clicked, this, &KDatePicker::todayButtonClicked);
     connect(d->selectMonth, &QAbstractButton::clicked, this, &KDatePicker::selectMonthClicked);
     connect(d->selectYear, &QAbstractButton::toggled, this, &KDatePicker::selectYearClicked);
@@ -331,7 +342,7 @@ void KDatePicker::initWidget(const QDate &date_)
     topLayout->addWidget(d->table);
 
     QBoxLayout *bottomLayout = new QHBoxLayout();
-    bottomLayout->setMargin(0);
+    bottomLayout->setContentsMargins(0, 0, 0, 0);
     bottomLayout->setSpacing(0);
     topLayout->addLayout(bottomLayout);
 
@@ -340,23 +351,20 @@ void KDatePicker::initWidget(const QDate &date_)
     bottomLayout->addWidget(d->selectWeek);
 
     d->table->setDate(date_);
-    dateChangedSlot(date_);    // needed because table emits changed only when newDate != oldDate
+    dateChangedSlot(date_); // needed because table emits changed only when newDate != oldDate
 }
 
-KDatePicker::~KDatePicker()
-{
-    delete d;
-}
+KDatePicker::~KDatePicker() = default;
 
 bool KDatePicker::eventFilter(QObject *o, QEvent *e)
 {
     if (e->type() == QEvent::KeyPress) {
         QKeyEvent *k = (QKeyEvent *)e;
 
-        if ((k->key() == Qt::Key_PageUp) ||
-                (k->key() == Qt::Key_PageDown) ||
-                (k->key() == Qt::Key_Up) ||
-                (k->key() == Qt::Key_Down)) {
+        if ((k->key() == Qt::Key_PageUp) //
+            || (k->key() == Qt::Key_PageDown) //
+            || (k->key() == Qt::Key_Up) //
+            || (k->key() == Qt::Key_Down)) {
             QApplication::sendEvent(d->table, e);
             d->table->setFocus();
             return true; // eat event
@@ -372,7 +380,7 @@ void KDatePicker::resizeEvent(QResizeEvent *e)
 
 void KDatePicker::dateChangedSlot(const QDate &date_)
 {
-    d->line->setText(locale().toString(date_, QLocale::ShortFormat));
+    d->line->setText(locale().toString(date_, dateFormatWith4DigitYear(locale(), QLocale::ShortFormat)));
     d->selectMonth->setText(locale().standaloneMonthName(date_.month(), QLocale::LongFormat));
     d->fillWeeksCombo();
 
@@ -383,17 +391,18 @@ void KDatePicker::dateChangedSlot(const QDate &date_)
     // the earliestValidDate as the first day.
     // In particular covers the case of Gregorian where 1/1/-4713 is not a valid QDate
     d->selectWeek->setCurrentIndex((date_.dayOfYear() + firstDay.dayOfWeek() - 2) / 7);
-    d->selectYear->setText(QString::number(date_.year()).rightJustified(4, QLatin1Char('0')));
+    d->selectYear->setText(locale().toString(date_, QStringLiteral("yyyy")).rightJustified(4, QLatin1Char('0')));
 
-    emit(dateChanged(date_));
+    Q_EMIT dateChanged(date_);
 }
 
 void KDatePicker::tableClickedSlot()
 {
-    emit(dateSelected(date()));
-    emit(tableClicked());
+    Q_EMIT dateSelected(date());
+    Q_EMIT tableClicked();
 }
 
+// TODO KF6: remove the const & from the returned QDate
 const QDate &KDatePicker::date() const
 {
     return d->table->date();
@@ -408,7 +417,7 @@ bool KDatePicker::setDate(const QDate &date_)
 
 void KDatePicker::monthForwardClicked()
 {
-    if (! setDate(date().addMonths(1))) {
+    if (!setDate(date().addMonths(1))) {
         QApplication::beep();
     }
     d->table->setFocus();
@@ -416,7 +425,7 @@ void KDatePicker::monthForwardClicked()
 
 void KDatePicker::monthBackwardClicked()
 {
-    if (! setDate(date().addMonths(-1))) {
+    if (!setDate(date().addMonths(-1))) {
         QApplication::beep();
     }
     d->table->setFocus();
@@ -424,7 +433,7 @@ void KDatePicker::monthBackwardClicked()
 
 void KDatePicker::yearForwardClicked()
 {
-    if (! setDate(d->table->date().addYears(1))) {
+    if (!setDate(d->table->date().addYears(1))) {
         QApplication::beep();
     }
     d->table->setFocus();
@@ -432,7 +441,7 @@ void KDatePicker::yearForwardClicked()
 
 void KDatePicker::yearBackwardClicked()
 {
-    if (! setDate(d->table->date().addYears(-1))) {
+    if (!setDate(d->table->date().addYears(-1))) {
         QApplication::beep();
     }
     d->table->setFocus();
@@ -442,7 +451,7 @@ void KDatePicker::weekSelected(int index)
 {
     QDate targetDay = d->selectWeek->itemData(index).toDateTime().date();
 
-    if (! setDate(targetDay)) {
+    if (!setDate(targetDay)) {
         QApplication::beep();
     }
     d->table->setFocus();
@@ -455,13 +464,13 @@ void KDatePicker::selectMonthClicked()
 
     QMenu popup(d->selectMonth);
     // Populate the pick list with all the month names, this may change by year
-    // JPL do we need to do somethng here for months that fall outside valid range?
+    // JPL do we need to do something here for months that fall outside valid range?
     const int monthsInYear = QDate(thisDate.year() + 1, 1, 1).addDays(-1).month();
     for (int m = 1; m <= monthsInYear; m++) {
         popup.addAction(locale().standaloneMonthName(m))->setData(m);
     }
 
-    QAction *item = popup.actions()[ thisDate.month() - 1 ];
+    QAction *item = popup.actions()[thisDate.month() - 1];
     // if this happens the above should already given an assertion
     if (item) {
         popup.setActiveAction(item);
@@ -481,7 +490,7 @@ void KDatePicker::selectMonthClicked()
     newDate.setDate(newDate.year(), newDate.month(), qMin(thisDate.day(), newDate.daysInMonth()));
 
     // Set the date, if it's invalid in any way then alert user and don't update
-    if (! setDate(newDate)) {
+    if (!setDate(newDate)) {
         QApplication::beep();
     }
 }
@@ -497,10 +506,10 @@ void KDatePicker::selectYearClicked()
     KPopupFrame *popup = new KPopupFrame(this);
     KDatePickerPrivateYearSelector *picker = new KDatePickerPrivateYearSelector(date(), popup);
     picker->resize(picker->sizeHint());
-    picker->setYear(thisDate.year());
+    picker->setYear(thisDate);
     picker->selectAll();
     popup->setMainWidget(picker);
-    connect(picker, SIGNAL(closeMe(int)), popup, SLOT(close(int)));
+    connect(picker, &KDatePickerPrivateYearSelector::closeMe, popup, &KPopupFrame::close);
     picker->setFocus();
 
     if (popup->exec(d->selectYear->mapToGlobal(QPoint(0, d->selectMonth->height())))) {
@@ -513,7 +522,7 @@ void KDatePicker::selectYearClicked()
         newDate = QDate(newDate.year(), newDate.month(), qMin(thisDate.day(), newDate.daysInMonth()));
 
         // Set the date, if it's invalid in any way then alert user and don't update
-        if (! setDate(newDate)) {
+        if (!setDate(newDate)) {
             QApplication::beep();
         }
     }
@@ -544,10 +553,10 @@ KDateTable *KDatePicker::dateTable() const
 
 void KDatePicker::lineEnterPressed()
 {
-    QDate newDate = QDate::fromString(d->line->text(), locale().dateFormat());
+    const QDate newDate = d->val->toDate(d->line->text());
 
     if (newDate.isValid()) {
-        emit(dateEntered(newDate));
+        Q_EMIT dateEntered(newDate);
         setDate(newDate);
         d->table->setFocus();
     } else {
@@ -572,16 +581,14 @@ void KDatePicker::setFontSize(int s)
         d->selectMonth,
         d->selectYear,
     };
-    const int NoOfButtons = sizeof(buttons) / sizeof(buttons[0]);
-    int count;
     QFont font;
     QRect r;
     // -----
     d->fontsize = s;
-    for (count = 0; count < NoOfButtons; ++count) {
-        font = buttons[count]->font();
+    for (QWidget *button : buttons) {
+        font = button->font();
         font.setPointSize(s);
-        buttons[count]->setFont(font);
+        button->setFont(font);
     }
     d->table->setFontSize(s);
 
@@ -606,18 +613,16 @@ void KDatePicker::setFontSize(int s)
 
     QStyleOptionToolButton opt;
     opt.initFrom(d->selectMonth);
-    opt.text      = longestMonth;
+    opt.text = longestMonth;
 
     // stolen from QToolButton
     QSize textSize = metrics.size(Qt::TextShowMnemonic, longestMonth);
-    textSize.setWidth(textSize.width() + metrics.width(QLatin1Char(' ')) * 2);
+    textSize.setWidth(textSize.width() + metrics.horizontalAdvance(QLatin1Char(' ')) * 2);
     int w = textSize.width();
     int h = textSize.height();
-    opt.rect.setHeight(h);   // PM_MenuButtonIndicator depends on the height
+    opt.rect.setHeight(h); // PM_MenuButtonIndicator depends on the height
 
-    QSize metricBound = style()->sizeFromContents(
-                            QStyle::CT_ToolButton, &opt, QSize(w, h), d->selectMonth
-                        ).expandedTo(QApplication::globalStrut());
+    QSize metricBound = style()->sizeFromContents(QStyle::CT_ToolButton, &opt, QSize(w, h), d->selectMonth);
 
     d->selectMonth->setMinimumSize(metricBound);
 }
@@ -636,13 +641,12 @@ void KDatePicker::setCloseButton(bool enable)
     if (enable) {
         d->closeButton = new QToolButton(this);
         d->closeButton->setAutoRaise(true);
-        const int spacingHint = style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
-        d->navigationLayout->addSpacing(spacingHint);
+        const int horizontalSpacing = style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing);
+        d->navigationLayout->addSpacing(horizontalSpacing);
         d->navigationLayout->addWidget(d->closeButton);
         d->closeButton->setToolTip(tr("Close", "@action:button"));
         d->closeButton->setIcon(QIcon::fromTheme(QStringLiteral("window-close")));
-        connect(d->closeButton, &QAbstractButton::clicked,
-                topLevelWidget(), &QWidget::close);
+        connect(d->closeButton, &QAbstractButton::clicked, topLevelWidget(), &QWidget::close);
     } else {
         delete d->closeButton;
         d->closeButton = nullptr;
@@ -655,3 +659,5 @@ bool KDatePicker::hasCloseButton() const
 {
     return (d->closeButton);
 }
+
+#include "kdatepicker.moc"
