@@ -11,6 +11,7 @@
 #include <kwidgetsaddons_export.h>
 
 #include <QWidget>
+#include <memory>
 
 /**
  * @class KXYSelector kxyselector.h KXYSelector
@@ -68,9 +69,9 @@ public:
     void setRange(int minX, int minY, int maxX, int maxY);
 
     /**
-      * Sets the color used to draw the marker
+     * Sets the color used to draw the marker
      * @param col the color
-      */
+     */
     void setMarkerColor(const QColor &col);
 
     /**
@@ -125,12 +126,11 @@ protected:
     void valuesFromPosition(int x, int y, int &xVal, int &yVal) const;
 
 private:
-    void setPosition(int xp, int yp);
+    KWIDGETSADDONS_NO_EXPORT void setPosition(int xp, int yp);
 
 private:
-    class Private;
-    friend class Private;
-    Private *const d;
+    friend class KXYSelectorPrivate;
+    std::unique_ptr<class KXYSelectorPrivate> const d;
 
     Q_DISABLE_COPY(KXYSelector)
 };
